library(ggplot2)
library(dplyr)

maxForce <- 1000 # Newtons
distance <- 1 # meter

tStep <- 0.0001 # seconds
minInertia = 10 # kg

# Constant force function
constantForce <- function(inertia = 10,
                          maxForce = 1000,
                          distance = 1,
                          tStep = 0.001,
                          gravity = 0) {
    currentDist <- 0
    currentVel <- 0
    timeVar <- 0
    
    while(currentDist < distance) {
        force <- maxForce - gravity
        if (force < 0) force <- 0        
        acceleration <- force / inertia
        currentVel <- currentVel + acceleration * tStep
        currentDist <- currentDist + currentVel * tStep
        timeVar <- timeVar + tStep    
    }
    
    return(distance / timeVar)
}

# Explosive Force 
explosiveForce <- function(inertia = 10,
                           maxForce = 1000,
                           distance = 1,
                           tStep = 0.001,
                           expFactor = 4,
                           gravity = 0) {
    currentDist <- 0
    currentVel <- 0
    timeVar <- 0
    force <- 0
    while(currentDist < distance) {
        force <- maxForce * (1 - exp(-(expFactor*timeVar)))
        force <- force - gravity
        if (force < 0) force <- 0
        acceleration <- force / inertia
        currentVel <- currentVel + acceleration * tStep
        currentDist <- currentDist + currentVel * tStep
        timeVar <- timeVar + tStep    
    }
    
    return(distance / timeVar)    
}

# Viscosive Force 
viscosiveForce <- function(inertia = 10,
                           maxForce = 1000,
                           distance = 1,
                           tStep = 0.001,
                           viscFactor = 150,
                           gravity = 0) {
    currentDist <- 0
    currentVel <- 0
    timeVar <- 0
    force <- 0
    while(currentDist < distance) {
        force <- maxForce - (currentVel * viscFactor)
        force <- force - gravity
        if (force < 0) force <- 0
        acceleration <- force / inertia
        currentVel <- currentVel + acceleration * tStep
        currentDist <- currentDist + currentVel * tStep
        timeVar <- timeVar + tStep    
    }
    
    return(distance / timeVar)    
}

# ViscoExplosive Force 
viscoExpForce <- function(inertia = 10,
                           maxForce = 1000,
                           distance = 1,
                           tStep = 0.001,
                           viscFactor = 1,
                           expFactor = 4,
                           gravity = 0) {
    currentDist <- 0
    currentVel <- 0
    timeVar <- 0
    force <- 0
    while(currentDist < distance) {
        force <- maxForce * (1 - exp(-(expFactor*timeVar)))
        force <- force - (currentVel * viscFactor)
        force <- force - gravity
        if (force < 0) force <- 0
        acceleration <- force / inertia
        currentVel <- currentVel + acceleration * tStep
        currentDist <- currentDist + currentVel * tStep
        timeVar <- timeVar + tStep    
    }
    
    return(distance / timeVar)    
}

#####
##### Main Loops
#####
loadVelocity <- data.frame(Load = numeric(0),
                           Type = character(0),
                           Simulation = character(0),
                           MeanVelocity = numeric(0),
                           stringsAsFactors = FALSE)

i <- 1
for (inertia in seq(minInertia,
                    0.9 * (maxForce / 9.81),
                    1)) {
    ## Only Inertia
    
    # Constant Force
    MV <- constantForce(inertia = inertia,
                        maxForce = maxForce,
                        distance = distance,
                        tStep = tStep)
    
    df <- data.frame(Load = inertia,
                     Type = "Inertia",
                     Simulation = "Constant Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)
    
    # Explosive Force
    MV <- explosiveForce(inertia = inertia,
                        maxForce = maxForce,
                        distance = distance,
                        tStep = tStep)
    
    df <- data.frame(Load = inertia,
                     Type = "Inertia",
                     Simulation = "Explosive Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)

    # Viscosive Force
    MV <- viscosiveForce(inertia = inertia,
                         maxForce = maxForce,
                         distance = distance,
                         tStep = tStep,
                         viscFactor = 150)
    
    df <- data.frame(Load = inertia,
                     Type = "Inertia",
                     Simulation = "Viscosive Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)   
    
    # Visco Exposive Force
    MV <- viscoExpForce(inertia = inertia,
                        maxForce = maxForce,
                        distance = distance,
                        tStep = tStep,
                        viscFactor = 150)
    
    df <- data.frame(Load = inertia,
                     Type = "Inertia",
                     Simulation = "Visco Explosive Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)    

    ## Inertia With Gravity
    
    # Constant Force
    MV <- constantForce(inertia = inertia,
                        maxForce = maxForce,
                        distance = distance,
                        tStep = tStep,
                        gravity = 9.81 * inertia)
    
    df <- data.frame(Load = inertia,
                     Type = "I + F",
                     Simulation = "Constant Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)
    
    # Explosive Force
    MV <- explosiveForce(inertia = inertia,
                         maxForce = maxForce,
                         distance = distance,
                         tStep = tStep,
                         gravity = 9.81 * inertia)
    
    df <- data.frame(Load = inertia,
                     Type = "I + F",
                     Simulation = "Explosive Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)
    
    # Viscosive Force
    MV <- viscosiveForce(inertia = inertia,
                         maxForce = maxForce,
                         distance = distance,
                         tStep = tStep,
                         gravity = 9.81 * inertia,
                         viscFactor = 150)
    
    df <- data.frame(Load = inertia,
                     Type = "I + F",
                     Simulation = "Viscosive Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)   
    
    # Visco Exposive Force
    MV <- viscoExpForce(inertia = inertia,
                        maxForce = maxForce,
                        distance = distance,
                        tStep = tStep,
                        gravity = 9.81 * inertia,
                        viscFactor = 150)
    
    df <- data.frame(Load = inertia,
                     Type = "I + F",
                     Simulation = "Visco Explosive Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)    
    
    
    ## Only force
    
    # Constant Force
    MV <- constantForce(inertia = minInertia,
                        maxForce = maxForce,
                        distance = distance,
                        tStep = tStep,
                        gravity = 9.81 * inertia)
    
    df <- data.frame(Load = inertia,
                     Type = "Force",
                     Simulation = "Constant Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)
    
    # Explosive Force
    MV <- explosiveForce(inertia = minInertia,
                         maxForce = maxForce,
                         distance = distance,
                         tStep = tStep,
                         gravity = 9.81 * inertia)
    
    df <- data.frame(Load = inertia,
                     Type = "Force",
                     Simulation = "Explosive Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)
    
    # Viscosive Force
    MV <- viscosiveForce(inertia = minInertia,
                         maxForce = maxForce,
                         distance = distance,
                         tStep = tStep,
                         gravity = 9.81 * inertia,
                         viscFactor = 150)
    
    df <- data.frame(Load = inertia,
                     Type = "Force",
                     Simulation = "Viscosive Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)   
    
    # Visco Exposive Force
    MV <- viscoExpForce(inertia = minInertia,
                        maxForce = maxForce,
                        distance = distance,
                        tStep = tStep,
                        gravity = 9.81 * inertia,
                        viscFactor = 150)
    
    df <- data.frame(Load = inertia,
                     Type = "Force",
                     Simulation = "Visco Explosive Force",
                     MeanVelocity = MV,
                     stringsAsFactors = FALSE)
    loadVelocity <- rbind(loadVelocity, df)  
    # Increment
    i <- i + 1
}

# Plot curves
gg <- ggplot(loadVelocity,
             aes(x = Load,
                 y = MeanVelocity,
                 color = Type)) +
       geom_line() + 
       facet_wrap(~Simulation, scales = "free") +
       theme_bw() +
       theme(legend.position = "top") +
       ylab("Mean Velocity (m/s)") +
       xlab("Load (kg)") + 
       labs(color ='Load type') 
gg


gg <- ggplot(loadVelocity,
             aes(x = Load,
                 y = MeanVelocity,
                 color = Simulation)) +
    geom_line() + 
    facet_wrap(~Type) +
    theme_bw() +
    theme(legend.position = "top") +
    ylab("Mean Velocity (m/s)") +
    xlab("Load (kg)") + 
    labs(color ='Load type') 
gg
